<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2015 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/
defined( '_JEXEC' ) or die( 'Restricted access' );

class RseventsproModelEvent extends JModelAdmin
{
	protected $text_prefix = 'COM_RSEVENTSPRO';
	
	/**
	 * Returns a Table object, always creating it.
	 *
	 * @param	type	The table type to instantiate
	 * @param	string	A prefix for the table class name. Optional.
	 * @param	array	Configuration array for model. Optional.
	 *
	 * @return	JTable	A database object
	*/
	public function getTable($type = 'Event', $prefix = 'RseventsproTable', $config = array()) {
		return JTable::getInstance($type, $prefix, $config);
	}
	
	/**
	 * Method to get a single record.
	 *
	 * @param	integer	The id of the primary key.
	 *
	 * @return	mixed	Object on success, false on failure.
	 */
	public function getItem($pk = null) {
		if ($item = parent::getItem($pk)) {
			$db = JFactory::getDbo();
			
			if (empty($item->location))
				$item->location = 0;
			
			$item->locationname = $this->getLocationName($item->location);
			
			// Convert image properties
			try {
				$registry = new JRegistry;
				$registry->loadString($item->properties);
				$item->properties = $registry->toArray();
			} catch (Exception $e) {
				$item->properties = array();
			}
			
			if (empty($item->start) || $item->start == $db->getNullDate()) {
				$item->start = JFactory::getDate()->toSql();
			}
			
			if (empty($item->end) || $item->end == $db->getNullDate()) {
				$end = JFactory::getDate();
				$end->modify('+2 hours');
				$item->end = $end->toSql();
			}
			
			if (empty($item->id)) 
				$item->published = 1;
			
			if (empty($item->URL) && empty($item->id)) 
				$item->URL = 'http://';
			
			if (empty($item->itemid)) 
				$item->itemid = '';
			
			if (empty($item->rsvp_quota)) 
				$item->rsvp_quota = '';
			
			if (empty($item->repeat_end) || $item->repeat_end == $db->getNullDate()) 
				$item->repeat_end = '';
			
			$item->repeat_end = str_replace(' 00:00:00','',$item->repeat_end);
			
			if (empty($item->start_registration) || $item->start_registration == $db->getNullDate()) 
				$item->start_registration = '';
			
			if (empty($item->end_registration) || $item->end_registration == $db->getNullDate()) 
				$item->end_registration = '';
				
			if (empty($item->unsubscribe_date) || $item->unsubscribe_date == $db->getNullDate()) 
				$item->unsubscribe_date = '';
				
			if (empty($item->early_fee_end) || $item->early_fee_end == $db->getNullDate()) 
				$item->early_fee_end = '';
				
			if (empty($item->late_fee_start) || $item->late_fee_start == $db->getNullDate()) 
				$item->late_fee_start = '';
				
			if (empty($item->rsvp_start) || $item->rsvp_start == $db->getNullDate()) 
				$item->rsvp_start = '';
			
			if (empty($item->rsvp_end) || $item->rsvp_end == $db->getNullDate()) 
				$item->rsvp_end = '';
			
			if (empty($item->waitinglist_limit)) 
				$item->waitinglist_limit = '';
				
			if (empty($item->repeat_type)) 
				$item->repeat_type = 1;
				
			if (empty($item->repeat_interval)) 
				$item->repeat_interval = 0;	
				
			if (empty($item->owner)) 
				$item->owner = JFactory::getUser()->get('id');
				
			if (empty($item->invoice_padding)) 
				$item->invoice_padding = '';
			
			$item->waitinglist_time = rseventsproHelper::secondsToTime($item->waitinglist_time);
			
			if (empty($item->tickets_amount))
				$item->tickets_amount = '';
			
			if (empty($item->overbooking_amount))
				$item->overbooking_amount = '';
			
			if (empty($item->max_tickets_amount))
				$item->max_tickets_amount = '';
			
			if (!empty($item->rsm_lists)) {
				try {
					$registry = new JRegistry;
					$registry->loadString($item->rsm_lists);
					$item->rsm_lists = $registry->toArray();
				} catch (Exception $e) {
					$item->rsm_lists = array();
				}
			}
			
			if (!empty($item->repeat_also)) {
				try {
					$registry = new JRegistry;
					$registry->loadString($item->repeat_also);
					$item->repeat_also = $registry->toArray();
				} catch (Exception $e) {
					$item->repeat_also = array();
				}
			}
			
			if (!empty($item->exclude_dates)) {
				try {
					$registry = new JRegistry;
					$registry->loadString($item->exclude_dates);
					$item->exclude_dates = $registry->toArray();
				} catch (Exception $e) {
					$item->exclude_dates = array();
				}
			}
			
			require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
			$event	= RSEvent::getInstance($item->id);
			
			$item->options = $event->getEventOptions();
			
			RSEventsProTranslations::overrideValues($item, 'event');
		}
		
		return $item;
	}
	
	/**
	 * Method to get the record form.
	 *
	 * @param	array	$data		Data for the form.
	 * @param	boolean	$loadData	True if the form is to load its own data (default case), false if not.
	 *
	 * @return	mixed	A JForm object on success, false on failure
	 * @since	1.6
	 */
	public function getForm($data = array(), $loadData = true) {
		// Get the form.
		$form = $this->loadForm('com_rseventspro.event', JPATH_ADMINISTRATOR.'/components/com_rseventspro/models/forms/event.xml', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form))
			return false;
		
		if (rseventsproHelper::isJ4()) {
			$form->setFieldAttribute('payments', 'layout', 'joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('rsm_lists', 'layout', 'joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('gallery_tags', 'layout', 'joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('repeat_also', 'layout', 'joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('repeat_also', 'class', 'repeat_also_fancy');
			$form->setFieldAttribute('exclude_dates', 'layout', 'joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('exclude_dates', 'class', 'exclude_dates_fancy');
		} else {
			$form->setFieldAttribute('name', 'class', 'span10');
			$form->setFieldAttribute('small_description', 'class', 'span10');
			$form->setFieldAttribute('URL', 'class', 'span10');
			$form->setFieldAttribute('phone', 'class', 'span10');
			$form->setFieldAttribute('email', 'class', 'span10');
			$form->setFieldAttribute('metaname', 'class', 'span10');
			$form->setFieldAttribute('metakeywords', 'class', 'span10');
			$form->setFieldAttribute('metadescription', 'class', 'span10');
			$form->setFieldAttribute('event_ended', 'class', 'span10');
			$form->setFieldAttribute('event_full', 'class', 'span10');
			$form->setFieldAttribute('repeat_on_day_order', 'class', 'input-small');
			$form->setFieldAttribute('repeat_on_day_type', 'class', 'input-small');
			$form->setFieldAttribute('repeat_interval', 'class', 'input-small');
			$form->setFieldAttribute('repeat_type', 'class', 'input-small');
			$form->setFieldAttribute('repeat_on_day', 'class', 'input-small');
			$form->setFieldAttribute('early_fee', 'class', 'input-small');
			$form->setFieldAttribute('early_fee_type', 'class', 'input-small');
			$form->setFieldAttribute('late_fee', 'class', 'input-small');
			$form->setFieldAttribute('late_fee_type', 'class', 'input-small');
			$form->setFieldAttribute('notify_me_emails', 'class', 'span10');
			$form->setFieldAttribute('notify_me_paid_emails', 'class', 'span10');
			$form->setFieldAttribute('notify_me_unsubscribe_emails', 'class', 'span10');
		}
		
		if ($form->getValue('allday') == 1) {
			$form->setFieldAttribute('start', 'allday', 'true');
			$form->setFieldAttribute('end', 'allday', 'true');
		}
		
		if ($form->getValue('rsvp') == 1) {
			$form->setFieldAttribute('registration', 'muted', 'true');
			$form->setFieldAttribute('registration', 'disabled', 'true');
		}
		
		if ($form->getValue('registration') == 1) {
			$form->setFieldAttribute('rsvp', 'muted', 'true');
			$form->setFieldAttribute('rsvp', 'disabled', 'true');
		}
		
		RSEventsProTranslations::setTranslateIcon($form, 'event');
		
		return $form;
	}
	
	public function getFormDependencies() {
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
		
		$id		= JFactory::getApplication()->input->getInt('id',0);
		$event	= RSEvent::getInstance($id);
		
		return $event->getEventFormDependencies();
	}
	
	public function getFormTickets($id = null) {
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
		
		$id		= is_null($id) ? JFactory::getApplication()->input->getInt('id',0) : $id;
		$event	= RSEvent::getInstance($id);
		
		return $event->getEventFormTickets();
	}
	
	public function getFormCoupons() {
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
		
		$id		= JFactory::getApplication()->input->getInt('id',0);
		$event	= RSEvent::getInstance($id);
		
		return $event->getEventFormCoupons();
	}
	
	/**
	 * Method to get the data that should be injected in the form.
	 *
	 * @return	mixed	The data for the form.
	 * @since	1.6
	 */
	protected function loadFormData() {
		if (JFactory::getApplication()->input->get('layout') == 'invoice') {
			return $this->getInvoice();
		}
		
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_rseventspro.edit.event.data', array());

		if (empty($data))
			$data = $this->getItem();

		return $data;
	}
	
	/**
	 * Method to save the form data.
	 *
	 * @param   array  $data  The form data.
	 *
	 * @return  boolean  True on success.
	 *
	 * @since   1.6
	 */
	public function save($data) {
		$table		= $this->getTable();
		$pk 		= (!empty($data['id'])) ? $data['id'] : (int) $this->getState($this->getName() . '.id');
		$isNew 		= true;
		$status		= false;
		
		// Load the row if saving an existing tag.
		if ($pk > 0) {
			$table->load($pk);
			$isNew  = false;
			$status = $table->published;
		}
		
		// Verify data
		if (!$table->verify($data)) {
			$this->setError($table->getError());
			return false;
		}
		
		// Bind the data.
		if (!$table->bind($data)) {
			$this->setError($table->getError());
			return false;
		}
		
		if (isset($data['from']))
			$table->from = $data['from'];
		
		// Check the data.
		if (!$table->check()) {
			$this->setError($table->getError());
			return false;
		}
		
		JFactory::getApplication()->triggerEvent('onrsepro_beforeEventStore',array(array('data'=>&$table)));
		
		// Store the data.
		if (!$table->store()) {
			$this->setError($table->getError());
			return false;
		}
		
		// After store
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
		$event = RSEvent::getInstance($table->id);
		$event->save($table, $isNew);
		
		JFactory::getApplication()->triggerEvent('onrsepro_afterEventStore',array(array('data'=>&$table, 'event' => $event)));
		
		// Check for cancel event, and send emails
		if ($status !== false) {
			if ($status != $data['published'] && $data['published'] == 3) {
				rseventsproEmails::cancel($table->id);
			}
		}
		
		// Replace translations
		if ($translation = RSEventsProTranslations::getTranslation('event', $table->id, 'name')) {
			$table->name = $translation;
		}
		
		$this->setState($this->getName() . '.id', $table->id);
		$this->setState($this->getName() . '.name', $table->name);
		
		return true;
	}
	
	/**
	 * Method to export events to iCal format.
	 *
	 * @return	.ics file
	 */
	public function exportical($pks) {
		if (!empty($pks)) {
			require_once JPATH_SITE.'/components/com_rseventspro/helpers/ical.php';
			$ical = RSEventsProiCal::getInstance($pks);
			
			$ical->toIcal();
		} else {
			$this->setError(JText::_('JLIB_DATABASE_ERROR_NO_ROWS_SELECTED'));
			return false;
		}
	}
	
	/**
	 * Method to export events to CSV format.
	 *
	 * @return	.csv file
	 */
	public function exportcsv($pks) {
		if (!empty($pks)) {
			$db = JFactory::getDbo();
			$query = $db->getQuery(true);
			$csv = '';
			
			$query->clear()
				->select('e.*')
				->select($db->qn('l.name','locationname'))->select($db->qn('l.address'))
				->from($db->qn('#__rseventspro_events','e'))
				->join('left', $db->qn('#__rseventspro_locations','l').' ON '.$db->qn('l.id').' = '.$db->qn('e.location'))
				->where($db->qn('e.id').' IN ('.implode(',',$pks).')');
			
			$db->setQuery($query);
			if ($events = $db->loadObjectList()) {
				foreach ($events as $event)
				{
					$name = strip_tags($event->name);
					$name = str_replace(array('\\r','\\n','\\t','"'),array("\015","\012","\011",'""'),$name);
					$description = strip_tags($event->description);
					$description = str_replace(array('\\r','\\n','\\t','"'),array("\015","\012","\011",'""'),$description);
					$url = strip_tags($event->URL);
					$url = str_replace(array('\\r','\\n','\\t','"'),array("\015","\012","\011",'""'),$url);
					$email = strip_tags($event->email);
					$email = str_replace(array('\\r','\\n','\\t','"'),array("\015","\012","\011",'""'),$email);
					$phone = strip_tags($event->phone);
					$phone = str_replace(array('\\r','\\n','\\t','"'),array("\015","\012","\011",'""'),$phone);
					$locationname = strip_tags($event->locationname);
					$locationname = str_replace(array('\\r','\\n','\\t','"'),array("\015","\012","\011",'""'),$locationname);
					$address = strip_tags($event->address);
					$address = str_replace(array('\\r','\\n','\\t','"'),array("\015","\012","\011",'""'),$address);
					
					$start	= JFactory::getDate($event->start);
					$end	= JFactory::getDate($event->end);
					
					$csv .= '"'.rseventsproHelper::fixValue($name).'","'.rseventsproHelper::fixValue($start->toSql()).'","'.rseventsproHelper::fixValue($end->toSql()).'","'.rseventsproHelper::fixValue($description).'","'.rseventsproHelper::fixValue($url).'","'.rseventsproHelper::fixValue($email).'","'.rseventsproHelper::fixValue($phone).'","'.rseventsproHelper::fixValue($locationname).'","'.rseventsproHelper::fixValue($address).'"'."\n";
				}
				
				$file = 'Events.csv';
				header("Content-type: text/csv; charset=UTF-8");
				header("Content-Disposition: attachment; filename=$file");
				echo rtrim($csv,"\n");
				JFactory::getApplication()->close();
			} else {
				$this->setError(JText::_('JLIB_DATABASE_ERROR_NO_ROWS_SELECTED'));
				return false;
			}
		} else {
			$this->setError(JText::_('JLIB_DATABASE_ERROR_NO_ROWS_SELECTED'));
			return false;
		}
	}
	
	/**
	 * Method to clear event rating.
	 *
	 * @return	boolean
	 */
	public function rating($pks) {
		if (!empty($pks)) {
			$db = JFactory::getDbo();
			$query = $db->getQuery(true);
			
			$query->clear()
				->delete()
				->from($db->qn('#__rseventspro_rating'))
				->where($db->qn('ide').' IN ('.implode(',',$pks).')');
			
			$db->setQuery($query);
			$db->execute();
			
			return true;
		} else {
			$this->setError(JText::_('JLIB_DATABASE_ERROR_NO_ROWS_SELECTED'));
			return false;
		}
	}
	
	/**
	 * Method to copy events.
	 *
	 * @return	void
	 */
	public function copy($pks) {
		if (!empty($pks)) {
			foreach ($pks as $pk) {
				rseventsproHelper::copy($pk,0);
			}
			return true;
		} else {
			$this->setError(JText::_('JLIB_DATABASE_ERROR_NO_ROWS_SELECTED'));
			return false;
		}
	}
	
	/**
	 * Method to remove ticket.
	 *
	 * @return	boolean
	 */
	public function removeticket() {
		$id = JFactory::getApplication()->input->getInt('id');
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$query->clear()
			->delete()
			->from($db->qn('#__rseventspro_tickets'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		$response = $db->execute();
		
		RSEventsProTranslations::removeTranslations('ticket', $id);
		
		if ($response)
			JFactory::getApplication()->triggerEvent('onrsepro_afterDeleteTicket', array(array('id' => $id)));
		
		return $response;
	}
	
	/**
	 * Method to remove coupon.
	 *
	 * @return	boolean
	 */
	public function removecoupon() {
		$id = JFactory::getApplication()->input->getInt('id');
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$query->clear()
			->delete()
			->from($db->qn('#__rseventspro_coupons'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		if ($db->execute()) {
			$query->clear()
				->delete()
				->from($db->qn('#__rseventspro_coupon_codes'))
				->where($db->qn('idc').' = '.$id);
			$db->setQuery($query);
			$db->execute();
			return true;
		} else return false;
	}
	
	/**
	 * Method to upload event icon.
	 *
	 * @return	boolean
	 */
	public function upload() {
		jimport('joomla.filesystem.file');
		
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$input	= JFactory::getApplication()->input;
		$id		= $input->getInt('id');
		$icon	= $input->files->get('icon',array(),'array');
		$local	= $input->getString('local');
		$path	= JPATH_SITE.'/components/com_rseventspro/assets/images/events/';
		$empty  = empty($local) && $icon['size'] == 0;
		$type	= '';
		$config = rseventsproHelper::getConfig();
		
		if ($icon['size'] > 0 && $icon['error'] == 0) {
			$type = 'upload';
		} elseif (!empty($local)) {
			$type = 'local';
		}
		
		if (!$empty || empty($type)) {
			// Check for file extension
			$ext = JFile::getExt($type == 'upload' ? $icon['name'] : $local);
			if (!in_array(strtolower($ext),array('jpg','png','jpeg'))) {
				$this->setError(JText::_('COM_RSEVENTSPRO_WRONG_FILE_TYPE'));
				return false;
			}
			
			// Remove old icon
			$query->clear()
				->select($db->qn('icon'))
				->from($db->qn('#__rseventspro_events'))
				->where($db->qn('id').' = '.$id);
			
			$db->setQuery($query);
			if ($eventicon = $db->loadResult()) {
				if (JFile::exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/'.$eventicon)) {
					JFile::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/'.$eventicon);
				}
				
				$extension	= JFile::getExt($eventicon);
				$name		= JFile::stripExt($eventicon);
				
				// Delete small icon
				if (file_exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/'.$config->icon_small_width.'/'.md5($config->icon_small_width.$name).'.'.$extension)) {
					JFile::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/'.$config->icon_small_width.'/'.md5($config->icon_small_width.$name).'.'.$extension);
				}
				
				// Delete big icon
				if (file_exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/'.$config->icon_big_width.'/'.md5($config->icon_big_width.$name).'.'.$extension)) {
					JFile::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/'.$config->icon_big_width.'/'.md5($config->icon_big_width.$name).'.'.$extension);
				}
				
				// Delete event listing icon from backend
				if (file_exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/70/'.md5('70'.$name).'.'.$extension)) {
					JFile::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/70/'.md5('70'.$name).'.'.$extension);
				}
				
				// Delete event edit icon
				if (file_exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/188/'.md5('188'.$name).'.'.$extension)) {
					JFile::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/188/'.md5('188'.$name).'.'.$extension);
				}
			}
			
			// Copy or upload the new icon
			if ($type == 'upload') {
				// Upload file
				if ($icon['error'] == 0) {
					$file		= JFile::makeSafe($icon['name']);
					$filename	= basename(JFile::stripExt($file));
					
					while(JFile::exists($path.$filename.'.'.$ext)) {
						$filename .= rand(1,999);
					}
						
					if (JFile::upload($icon['tmp_name'],$path.$filename.'.'.$ext)) {
						$query->clear()
							->update($db->qn('#__rseventspro_events'))
							->set($db->qn('icon').' = '.$db->q($filename.'.'.$ext))
							->set($db->qn('properties').' = '.$db->q(''))
							->where($db->qn('id').' = '.$id);
						
						$db->setQuery($query);
						$db->execute();
						
						$this->setState('com_rseventspro.edit.icon', $filename.'.'.$ext);
					} else { 
						$this->setError(JText::_('COM_RSEVENTSPRO_UPLOAD_ERROR'));
						return false;
					}
				} else {
					$this->setError(JText::_('COM_RSEVENTSPRO_FILE_ERROR'));
					return false;
				}
			} else {
				// Copy local file
				$params 	= JComponentHelper::getParams('com_media');
				$root		= JPATH_ROOT . '/' . $params->get('image_path', 'images');
				$root		= str_replace(DIRECTORY_SEPARATOR, '/', $root . '/');
				$file		= $root.$local;
				$filename	= basename(JFile::stripExt($file));
				
				while(JFile::exists($path.$filename.'.'.$ext)) {
					$filename .= rand(1,999);
				}
				
				if (JFile::copy($file,$path.$filename.'.'.$ext)) {
					$query->clear()
						->update($db->qn('#__rseventspro_events'))
						->set($db->qn('icon').' = '.$db->q($filename.'.'.$ext))
						->set($db->qn('properties').' = '.$db->q(''))
						->where($db->qn('id').' = '.$id);
					
					$db->setQuery($query);
					$db->execute();
					
					$this->setState('com_rseventspro.edit.icon', $filename.'.'.$ext);
				} else { 
					$this->setError(JText::_('COM_RSEVENTSPRO_COPY_ERROR'));
					return false;
				}
			}
			
		} else {
			$this->setError(JText::_('COM_RSEVENTSPRO_NO_FILE_SELECTED'));
			return false;
		}
		
		return true;
	}
	
	public function getIcon() {
		if ($icon = JFactory::getApplication()->input->getString('icon','')) {
			return base64_decode($icon);
		}
		
		return false;
	}
	
	/**
	 * Method to delete event icon.
	 *
	 * @return	boolean
	 */
	public function deleteicon() {
		jimport('joomla.filesystem.file');
		$id		= JFactory::getApplication()->input->getInt('id');
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('icon'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		if ($icon = $db->loadResult()) {
			if (JFile::exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/'.$icon))
				JFile::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/'.$icon);
			
			$query->clear()
				->update($db->qn('#__rseventspro_events'))
				->set($db->qn('icon').' = '.$db->q(''))
				->set($db->qn('properties').' = '.$db->q(''))
				->where($db->qn('id').' = '.$id);
			
			$db->setQuery($query);
			$db->execute();
		}
		return true;
	}
	
	/**
	 * Method to crop the event icon.
	 *
	 * @return	boolean
	 */
	public function crop() {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$input	= JFactory::getApplication()->input;
		$id		= $input->getInt('id');
		$path	= JPATH_SITE.'/components/com_rseventspro/assets/images/events/';
		$thumbs = JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/';
		
		$query->clear()
			->select($db->qn('icon'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		$icon = $db->loadResult();
		
		$this->setState('com_rseventspro.crop.icon', $icon);
		
		$left	= $input->getInt('x1');
		$top	= $input->getInt('y1');
		$width	= $input->getInt('width');
		$height	= $input->getInt('height');
		
		$properties = array('left' => $left, 'top' => $top, 'width' => $width, 'height' => $height);
		$registry = new JRegistry;
		$registry->loadArray($properties);
		$properties = $registry->toString();
		
		$query->clear()
			->update($db->qn('#__rseventspro_events'))
			->set($db->qn('properties').' = '.$db->q($properties))
			->set($db->qn('aspectratio').' = '.$db->q($input->getInt('aspectratio',0)))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		$db->execute();
		
		// Remove old thumbs
		jimport('joomla.filesystem.folder');
		jimport('joomla.filesystem.file');
		
		// Get file extension
		$extension	= JFile::getExt($icon);
		// Strip extension
		$name		= JFile::stripExt($icon);
		
		if ($folders = JFolder::folders($thumbs)) {
			$folders = array_map('intval',$folders);
			
			foreach ($folders as $folder) {
				if (file_exists($thumbs.$folder.'/'.md5($folder.$name).'.'.$extension)) {
					JFile::delete($thumbs.$folder.'/'.md5($folder.$name).'.'.$extension);
				}
			}
		}
		
		return true;
	}
	
	/**
	 * Method to save file details
	 *
	 * @return	boolean
	 */
	public function savefile() {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$jinput	= JFactory::getApplication()->input->post;
		$id		= $jinput->getInt('id');
		$permissions = '';
		
		$fp0 = $jinput->get('fp0');
		$fp1 = $jinput->get('fp1');
		$fp2 = $jinput->get('fp2');
		$fp3 = $jinput->get('fp3');
		$fp4 = $jinput->get('fp4');
		$fp5 = $jinput->get('fp5');
		
		if (isset($fp0) && $fp0 == 1) $permissions .= '1'; else $permissions .= '0';
		if (isset($fp1) && $fp1 == 1) $permissions .= '1'; else $permissions .= '0';
		if (isset($fp2) && $fp2 == 1) $permissions .= '1'; else $permissions .= '0';
		if (isset($fp3) && $fp3 == 1) $permissions .= '1'; else $permissions .= '0';
		if (isset($fp4) && $fp4 == 1) $permissions .= '1'; else $permissions .= '0';
		if (isset($fp5) && $fp5 == 1) $permissions .= '1'; else $permissions .= '0';
		
		$query->clear()
			->update($db->qn('#__rseventspro_files'))
			->set($db->qn('name').' = '.$db->q($jinput->getString('name')))
			->set($db->qn('permissions').' = '.$db->q($permissions))
			->where($db->qn('id').' = '.$db->q($id));
		
		$db->setQuery($query);
		$db->execute();
		
		$this->setState('com_rseventspro.file.id',$id);
		$this->setState('com_rseventspro.file.name',$jinput->getString('name'));
		
		return true;
	}
	
	/**
	 * Method to remove event files
	 *
	 * @return	boolean
	 */
	public function removefile() {
		jimport('joomla.filesystem.file');
		
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$id		= JFactory::getApplication()->input->getInt('id');
		
		$query->clear()
			->select($db->qn('location'))
			->from($db->qn('#__rseventspro_files'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		if ($file = $db->loadResult()) {
			if (JFile::exists(JPATH_SITE.'/components/com_rseventspro/assets/images/files/'.$file)) {
				if (JFile::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/files/'.$file)) {
					$query->clear()
						->delete()
						->from($db->qn('#__rseventspro_files'))
						->where($db->qn('id').' = '.$id);
						
					$db->setQuery($query);
					$db->execute();
					return true;
				}
			}
		}
		
		return false;
	}
	
	/**
	 * Method to get save tickets configuration
	 *
	 * @return	array
	 */
	public function tickets() {
		$db			= JFactory::getDbo();
		$query		= $db->getQuery(true);
		$input		= JFactory::getApplication()->input;
		$params		= $input->get('params',array(),'array');
		
		if (!empty($params)) {
			foreach ($params as $i => $param) {
				$registry = new JRegistry;
				$registry->loadArray($param);
				$position = $registry->toString();
				
				$query->clear()
					->update($db->qn('#__rseventspro_tickets'))
					->set($db->qn('position').' = '.$db->q($position))
					->where($db->qn('id').' = '.(int) $i);
				
				$db->setQuery($query);
				$db->execute();
			}
		}
	}
	
	/**
	 * Method to delete the reports.
	 */
	public function deletereports($pks) {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->delete()
			->from($db->qn('#__rseventspro_reports'))
			->where($db->qn('id').' IN ('.implode(',',$pks).')');
		$db->setQuery($query);
		$db->execute();
	}
	
	/**
	 * Method to toggle the featured setting of events.
	 *
	 * @param   array    The ids of the items to toggle.
	 * @param   integer  The value to toggle to.
	 *
	 * @return  boolean  True on success.
	 */
	public function featured($pks, $value = 0) {
		// Sanitize the ids.
		$pks = (array) $pks;
		$pks = array_map('intval',$pks);

		if (empty($pks)) {
			$this->setError(JText::_('JERROR_NO_ITEMS_SELECTED'));
			return false;
		}

		try {
			$db		= $this->getDbo();
			$query	= $db->getQuery(true);
			
			$query->update($db->qn('#__rseventspro_events'))
				->set($db->qn('featured').' = '.(int) $value)
				->where($db->qn('id').' IN ('.implode(',', $pks).')');
			
			$db->setQuery($query);
			$db->execute();
		} catch (Exception $e) {
			$this->setError($e->getMessage());
			return false;
		}

		return true;
	}
	
	public function batchProcess($pks) {
		// Sanitize the ids.
		$pks = (array) $pks;
		$pks = array_map('intval',$pks);
		
		$batch	 = JFactory::getApplication()->input->get('batch',array(),'array');
		$all	 = isset($batch['all']) ? $batch['all'] : 0;
		$options = isset($batch['options']) ? $batch['options'] : array();
		$enable	 = isset($batch['enable_options']) ? 1 : 0;
		
		$categories	= isset($batch['categories']) ? $batch['categories'] : array();
		$tags		= isset($batch['tags']) ? $batch['tags'] : array();
		$groups		= isset($batch['groups']) ? $batch['groups'] : array();
		$sponsors	= isset($batch['sponsors']) ? $batch['sponsors'] : array();
		$speakers	= isset($batch['speakers']) ? $batch['speakers'] : array();
		$location	= isset($batch['locations']) ? $batch['locations'] : 0;
		$itemid		= isset($batch['itemid']) ? $batch['itemid'] : '';
		$type		= isset($batch['type']) ? $batch['type'] : 'r';
		
		$registration 	= isset($batch['registration']) ? $batch['registration'] : '';
		$activation 	= isset($batch['activation']) ? $batch['activation'] : '';
		$denied			= isset($batch['denied']) ? $batch['denied'] : '';
		$refund 		= isset($batch['refund']) ? $batch['refund'] : '';
		
		if (empty($pks) && !$all) {
			$this->setError(JText::_('JERROR_NO_ITEMS_SELECTED'));
			return false;
		}
		
		try {
			$db		 = $this->getDbo();
			$query	 = $db->getQuery(true);
			
			if ($enable) {
				$defaults = rseventsproHelper::getDefaultOptions();
				
				try {
					$registry = new JRegistry;
					$registry->loadString($defaults);
					$defaults = $registry->toArray();
				} catch (Exception $e) {}
				
				foreach ($defaults as $name => $value) {
					if (!isset($options[$name]))
						$options[$name] = 0;
				}
			
				$registry = new JRegistry;
				$registry->loadArray($options);
			
				$query->update($db->qn('#__rseventspro_events'))->set($db->qn('options').' = '.$db->q($registry->toString()));
			
				if ($location) {
					$query->set($db->qn('location').' = '.$db->q($location));
				}
				
				if ($itemid) {
					$query->set($db->qn('itemid').' = '.$db->q($itemid));
				}
				
				if ($registration != '') {
					$query->set($db->qn('disable_registration').' = '.$db->q($registration));
				}
				
				if ($activation != '') {
					$query->set($db->qn('disable_activation').' = '.$db->q($activation));
				}
				
				if ($denied != '') {
					$query->set($db->qn('disable_denied').' = '.$db->q($denied));
				}
				
				if ($refund != '') {
					$query->set($db->qn('disable_refund').' = '.$db->q($refund));
				}
				
				if (!$all) {
					$query->where($db->qn('id').' IN ('.implode(',', $pks).')');
				}
				
				$db->setQuery($query);
				$db->execute();
			} else {
				if ($location || $itemid || $registration != '' || $activation != '' || $denied != '' || $refund != '') {
					$query->update($db->qn('#__rseventspro_events'));
					
					if ($location) {
						$query->set($db->qn('location').' = '.$db->q($location));
					}
					
					if ($itemid) {
						$query->set($db->qn('itemid').' = '.$db->q($itemid));
					}
					
					if ($registration != '') {
						$query->set($db->qn('disable_registration').' = '.$db->q($registration));
					}
					
					if ($activation != '') {
						$query->set($db->qn('disable_activation').' = '.$db->q($activation));
					}
					
					if ($denied != '') {
						$query->set($db->qn('disable_denied').' = '.$db->q($denied));
					}
					
					if ($refund != '') {
						$query->set($db->qn('disable_refund').' = '.$db->q($refund));
					}
					
					if (!$all) {
						$query->where($db->qn('id').' IN ('.implode(',', $pks).')');
					}
					
					$db->setQuery($query);
					$db->execute();
				}
			}
			
			// Update categories
			$this->updateBatchOptions($categories, 'category', $pks, $all, $type);
			// Update groups
			$this->updateBatchOptions($groups, 'groups', $pks, $all, $type);
			// Update sponsors
			$this->updateBatchOptions($sponsors, 'sponsor', $pks, $all, $type);
			// Update speakers
			$this->updateBatchOptions($speakers, 'speaker', $pks, $all, $type);
			
			// Update tags
			if (!empty($tags)) {
				// Get the tags ids
				$ids = array();
				foreach ($tags as $tag) {
					$tag = trim($tag);
					$query->clear()
						->select($db->qn('id'))
						->from($db->qn('#__rseventspro_tags'))
						->where($db->qn('name').' = '.$db->q($tag));
					$db->setQuery($query);
					if ($tid = (int) $db->loadResult()) {
						$ids[] = $tid;
					} else {
						$query->clear()
							->insert($db->qn('#__rseventspro_tags'))
							->set($db->qn('name').' = '.$db->q($tag))
							->set($db->qn('published').' = '.$db->q(1));
						
						$db->setQuery($query);
						$db->execute();
						$tid = $db->insertid();
						$ids[] = $tid;
					}
				}
				
				if ($all) {
					if ($type == 'r') {					
						$query->clear()
							->delete($db->qn('#__rseventspro_taxonomy'))
							->where($db->qn('type').' = '.$db->q('tag'));
						$db->setQuery($query);
						$db->execute();
					}
					
					$query->clear()
						->select($db->qn('id'))
						->from($db->qn('#__rseventspro_events'));
					$db->setQuery($query);
					if ($events = $db->loadColumn()) {
						foreach ($events as $id) {
							foreach ($ids as $idt) {
								$query->clear()
									->select($db->qn('id'))
									->from($db->qn('#__rseventspro_taxonomy'))
									->where($db->qn('type').' = '.$db->q('tag'))
									->where($db->qn('ide').' = '.$db->q($id))
									->where($db->qn('id').' = '.$db->q($idt));
								$db->setQuery($query);
								$txid = (int) $db->loadResult();
								
								if (!$txid) {
									$query->clear()
										->insert($db->qn('#__rseventspro_taxonomy'))
										->set($db->qn('type').' = '.$db->q('tag'))
										->set($db->qn('ide').' = '.$db->q($id))
										->set($db->qn('id').' = '.$db->q($idt));
									$db->setQuery($query);
									$db->execute();
								}
							}
						}
					}
				} else {
					if ($type == 'r') {
						$query->clear()
							->delete($db->qn('#__rseventspro_taxonomy'))
							->where($db->qn('type').' = '.$db->q('tag'))
							->where($db->qn('ide').' IN ('.implode(',', $pks).')');
						$db->setQuery($query);
						$db->execute();
					}
					
					foreach ($pks as $id) {
						foreach ($ids as $idt) {
							$query->clear()
								->select($db->qn('id'))
								->from($db->qn('#__rseventspro_taxonomy'))
								->where($db->qn('type').' = '.$db->q('tag'))
								->where($db->qn('ide').' = '.$db->q($id))
								->where($db->qn('id').' = '.$db->q($idt));
							$db->setQuery($query);
							$txid = (int) $db->loadResult();
							
							if (!$txid) {
								$query->clear()
									->insert($db->qn('#__rseventspro_taxonomy'))
									->set($db->qn('type').' = '.$db->q('tag'))
									->set($db->qn('ide').' = '.$db->q($id))
									->set($db->qn('id').' = '.$db->q($idt));
								$db->setQuery($query);
								$db->execute();
							}
						}
					}
				}
			}
			
			return true;
		} catch (Exception $e) {
			$this->setError($e->getMessage());
			return false;
		}
	}
	
	public function sync() {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$config	= JFactory::getConfig();
		$offset	= $config->get('offset');
		
		$query->select($db->qn('id'))
			->select($db->qn('start'))->select($db->qn('timezone'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('allday').' = 1');
		
		$db->setQuery($query);
		if ($events = $db->loadObjectList()) {
			foreach ($events as $event) {
				if ($event->timezone == $offset) {
					continue;
				}
				
				if (empty($event->timezone)) {
					$event->timezone = 'UTC';
				}
				
				$date = new DateTime($event->start, new DateTimezone('UTC'));
				$date->setTimezone(new DateTimezone($event->timezone));
				
				$date = new DateTime($date->format('Y-m-d H:i:s'), new DateTimezone($offset));
				$date->setTimezone(new DateTimezone('UTC'));
				
				$start = $date->format('Y-m-d H:i:s');
				
				$query->clear()
					->update($db->qn('#__rseventspro_events'))
					->set($db->qn('start').' = '.$db->q($start))
					->set($db->qn('timezone').' = '.$db->q($offset))
					->where($db->qn('id').' = '.$db->q($event->id));
				$db->setQuery($query);
				$db->execute();
			}
		}
		
		return true;
	}
	
	public function ticketsorder() {
		$db		 = JFactory::getDbo();
		$query	 = $db->getQuery(true);
		$input	 = JFactory::getApplication()->input;
		$id		 = $input->getInt('id',0);
		$tickets = $input->get('ticket', array(), 'array');
		
		foreach ($tickets as $i => $ticket) {
			$query->clear()
				->update($db->qn('#__rseventspro_tickets'))
				->set($db->qn('order').' = '.$db->q($i))
				->where($db->qn('id').' = '.$db->q($ticket))
				->where($db->qn('ide').' = '.$db->q($id));
			$db->setQuery($query);
			$db->execute();
		}
	}
	
	public function subscribeUsers() {
		if (!rseventsproHelper::isRsmail()) {
			return;
		}
		
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$app 	= JFactory::getApplication();
		$id		= $app->input->getInt('id',0);
		$lists 	= $app->input->get('lists', array(), 'array');
		$status	= $app->input->get('status', array(), 'array');
		
		if ($lists) {
			$query->select('DISTINCT '.$db->qn('u.email'))
				->select($db->qn('u.id'))
				->from($db->qn('#__rseventspro_users','u'))
				->where($db->qn('u.ide').' = '.$id);
			
			if ($status) {
				$status = array_map('intval', $status);
				$query->where($db->qn('u.state').' IN ('.implode(',', $status).')');
			}
			
			$app->triggerEvent('onrsepro_subscriptionsQuery',array(array('query'=>&$query, 'rule' => 'u.ide')));
			
			$db->setQuery($query);
			if ($subscribers = $db->loadObjectList()) {
				require_once JPATH_SITE.'/components/com_rsmail/helpers/actions.php';
				$rsmail = new rsmHelper;
				
				foreach ($subscribers as $subscriber) {
					foreach ($lists as $listID) {
						$query->clear()
							->select($db->qn('id'))
							->from($db->qn('#__rseventspro_rsmail'))
							->where($db->qn('ids').' = '.$db->q($subscriber->id))
							->where($db->qn('idl').' = '.$db->q($listID));
						$db->setQuery($query);
						if (!$db->loadResult()) {
							$list = (object) array('id' => $listID);
							if ($rsmail->subscribe($subscriber->email, $list, 1, 0)) {
								$query->clear()
									->insert($db->qn('#__rseventspro_rsmail'))
									->set($db->qn('ids').' = '.$db->q($subscriber->id))
									->set($db->qn('idl').' = '.$db->q($listID));
								$db->setQuery($query);
								$db->execute();
							}
						}
					}
				}
			}
		}
	}
	
	public function duplicateticket() {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$app	= JFactory::getApplication();
		$id		= $app->input->getInt('id', 0);
		$cart	= false;
		
		$query->clear()
			->select('*')
			->from($db->qn('#__rseventspro_tickets'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$ticket = $db->loadObject();
		
		$ticket->id = 0;
		$ticket->name = JText::sprintf('COM_RSEVENTSPRO_DUPLICATE_TICKET_NAME', $ticket->name);
		
		$query->clear()
			->select('MAX('.$db->qn('order').')')
			->from($db->qn('#__rseventspro_tickets'))
			->where($db->qn('ide').' = '.$ticket->ide);
		$db->setQuery($query);
		$ticket->order = (int) $db->loadResult() + 1;
		
		// Save ticket
		$db->insertObject('#__rseventspro_tickets', $ticket, 'id');
		
		$app->triggerEvent('onrsepro_isCart',array(array('cart'=>&$cart)));
		
		if ($cart) {
			$query->clear()
				->select('*')
				->from($db->qn('#__rseventspro_fields'))
				->where($db->qn('idt').' = '.$db->q($id));
			$db->setQuery($query);
			if ($fields = $db->loadObjectList()) {
				foreach ($fields as $field) {
					$field->id = 0;
					$field->idt = $ticket->id;
					$db->insertObject('#__rseventspro_fields', $field);
				}
			}
		}
		
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
		
		$eventClass = RSEvent::getInstance($ticket->ide);
		
		$app->input->set('format', 'raw');
		
		$view = new JViewLegacy(array(
			'name' => 'event',
			'layout' => 'edit',
			'base_path' => JPATH_ADMINISTRATOR.'/components/com_rseventspro'
		));
		
		$view->eventClass 	= $eventClass;
		$view->tickets 		= $eventClass->getTickets($ticket->id);
		$view->ticketsform	= $this->getFormTickets($ticket->ide);
		
		$response = new stdClass();
		$response->id = $ticket->id;
		$response->name = $ticket->name;
		$response->html = $view->loadTemplate('tickets');
		
		return $response;
	}
	
	public function save2copy($id) {
		return rseventsproHelper::copy($id,0);
	}
	
	public function cancelevent($pks) {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		// Sanitize the ids.
		$pks = (array) $pks;
		$pks = array_map('intval',$pks);
		
		if ($pks) {
			$query->clear()
				->update($db->qn('#__rseventspro_events'))
				->set($db->qn('published').' = '.$db->q(3))
				->where($db->qn('id').' IN ('.implode(',',$pks).')');
			$db->setQuery($query);
			$db->execute();
			
			// Send emails
			foreach ($pks as $id) {
				rseventsproEmails::cancel($id);
			}
		}
		
		return true;
	}
	
	protected function getLocationName($id) {
		static $locations = array();
		
		if (!isset($locations[$id])) {
			$db = JFactory::getDbo();
			$query = $db->getQuery(true);
			
			$query->clear()
				->select($db->qn('name'))
				->from($db->qn('#__rseventspro_locations'))
				->where($db->qn('id').' = '.$id);
			$db->setQuery($query);
			$name = $db->loadResult();
			
			if (JFactory::getApplication()->isClient('site')) {
				// Replace translations
				if ($translation = RSEventsProTranslations::getTranslation('location', $id, 'name')) {
					$name = $translation;
				}
			}
			
			$locations[$id] = $name;
		}
		
		return $locations[$id];
	}
	
	protected function updateBatchOptions($array, $taxonomy, $pks, $all, $type) {
		if (!empty($array)) {
			$db		= JFactory::getDbo();
			$query	= $db->getQuery(true);
			$array	= array_map('intval',$array);
			
			if ($all) {
				if ($type == 'r') {
					$query->clear()
						->delete($db->qn('#__rseventspro_taxonomy'))
						->where($db->qn('type').' = '.$db->q($taxonomy));
					$db->setQuery($query);
					$db->execute();
				}
				
				$query->clear()
					->select($db->qn('id'))
					->from($db->qn('#__rseventspro_events'));
				$db->setQuery($query);
				if ($events = $db->loadColumn()) {
					foreach ($events as $id) {
						foreach ($array as $arrayID) {
							$query->clear()
								->select($db->qn('id'))
								->from($db->qn('#__rseventspro_taxonomy'))
								->where($db->qn('type').' = '.$db->q($taxonomy))
								->where($db->qn('ide').' = '.$db->q($id))
								->where($db->qn('id').' = '.$db->q($arrayID));
							$db->setQuery($query);
							$txid = (int) $db->loadResult();
							
							if (!$txid) {
								$query->clear()
									->insert($db->qn('#__rseventspro_taxonomy'))
									->set($db->qn('type').' = '.$db->q($taxonomy))
									->set($db->qn('ide').' = '.$db->q($id))
									->set($db->qn('id').' = '.$db->q($arrayID));
								$db->setQuery($query);
								$db->execute();
							}
						}
					}
				}
			} else {
				if ($type == 'r') {
					$query->clear()
						->delete($db->qn('#__rseventspro_taxonomy'))
						->where($db->qn('type').' = '.$db->q($taxonomy))
						->where($db->qn('ide').' IN ('.implode(',', $pks).')');
					$db->setQuery($query);
					$db->execute();
				}
				
				foreach ($pks as $id) {
					foreach ($array as $arrayID) {
						$query->clear()
							->select($db->qn('id'))
							->from($db->qn('#__rseventspro_taxonomy'))
							->where($db->qn('type').' = '.$db->q($taxonomy))
							->where($db->qn('ide').' = '.$db->q($id))
							->where($db->qn('id').' = '.$db->q($arrayID));
						$db->setQuery($query);
						$txid = (int) $db->loadResult();
						
						if (!$txid) {
							$query->clear()
								->insert($db->qn('#__rseventspro_taxonomy'))
								->set($db->qn('type').' = '.$db->q($taxonomy))
								->set($db->qn('ide').' = '.$db->q($id))
								->set($db->qn('id').' = '.$db->q($arrayID));
							$db->setQuery($query);
							$db->execute();
						}
					}
				}
			}
		}
	}
	
	public function getTicketForm() {
		$form = JForm::getInstance('ticketoptions', JPATH_ADMINISTRATOR.'/components/com_rseventspro/models/forms/event_ticketoptions.xml', array('control' => 'jform'));
		$data = $this->getTicket();
		
		$form->bind($data);
		
		return $form;
	}
	
	public function getTicket() {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$id		= JFactory::getApplication()->input->getInt('id', 0);
		
		$query->clear()
			->select('*')
			->from($db->qn('#__rseventspro_tickets'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$ticket = $db->loadObject();
		
		if (empty($ticket->start)) $ticket->start = 1;
		if (empty($ticket->disabled)) $ticket->disabled = '';
		if (empty($ticket->available)) $ticket->available = 1;
		
		try {
			$registry = new JRegistry;
			$registry->loadString($ticket->position);
			$ticket->position = $registry->toArray();
		} catch (Exception $e) {
			$ticket->position = array();
		}
		
		try {
			$registry = new JRegistry;
			$registry->loadString($ticket->manual_disabled);
			$ticket->manual_disabled = $registry->toArray();
		} catch (Exception $e) {
			$ticket->manual_disabled = array();
		}
		
		return $ticket;
	}
	
	public function getPurchasedTickets() {
		$id	= JFactory::getApplication()->input->getInt('id', 0);
		
		return $this->ticketCount($id);
	}
	
	public function saveTicketOptions($data) {
		$data['start'] = (int) $data['start'];
		$data['disabled'] = (int) $data['disabled'];
		$data['available'] = (int) $data['available'];
		$reset = (int) $data['reset'];
		
		unset($data['reset']);
		
		if (isset($data['manual_disabled']) && is_array($data['manual_disabled'])) {
			$registry = new JRegistry;
			$registry->loadArray($data['manual_disabled']);
			$data['manual_disabled'] = $registry->toString();
		} else {
			$data['manual_disabled'] = '';
		}
		
		if ($reset) {
			$data['disabled'] = 0;
			$data['available'] = 1;
			$data['manual_disabled'] = '';
			unset($data['start']);
		} else {
			if ($this->ticketCount($data['id'])) {
				return false;
			}
		}
		
		$ticket = (object) $data;
		
		return JFactory::getDbo()->updateObject('#__rseventspro_tickets', $ticket, 'id');
	}
	
	protected function ticketCount($id) {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select('COUNT('.$db->qn('id').')')
			->from($db->qn('#__rseventspro_user_tickets'))
			->where($db->qn('idt').' = '.$db->q($id));
		$db->setQuery($query);
		return (int) $db->loadResult();
	}
	
	public function getInvoice() {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery();
		$id		= JFactory::getApplication()->input->getInt('id',0);
		$lang	= $this->getLanguage();
		
		$query->clear()
			->select($db->qn('title'))->select($db->qn('layout'))
			->from($db->qn('#__rseventspro_invoice'))
			->where($db->qn('ide'). ' = '.$db->q($id))
			->where($db->qn('lang'). ' = '.$db->q($lang));
		$db->setQuery($query);
		$invoice = $db->loadObject();
		
		if (!$invoice) {
			$query->clear()
				->select($db->qn('title'))->select($db->qn('layout'))
				->from($db->qn('#__rseventspro_invoice'))
				->where($db->qn('ide'). ' = '.$db->q($id))
				->where($db->qn('lang'). ' = '.$db->q('en-GB'));
			$db->setQuery($query);
			$invoice = $db->loadObject();
			
			if (!$invoice) {
				$invoice = new stdClass();
				$invoice->title = '';
				$invoice->layout = '';
			}
		}
		
		return array('invoice_language' => $lang, 'invoice_title' => $invoice->title, 'invoice_layout' => $invoice->layout);
	}
	
	public function getLanguage() {
		$jform	= JFactory::getApplication()->input->get('jform',array(),'array');
		return !empty($jform['invoice_language']) ? $jform['invoice_language'] : JFactory::getLanguage()->getTag();
	}
	
	public function saveinvoice($data) {
		$db			= JFactory::getDbo();
		$query		= $db->getQuery();
		$ide		= isset($data['id']) ? $data['id'] : 0;
		$language	= isset($data['invoice_language']) ? $data['invoice_language'] : $this->getLanguage();
		$title		= isset($data['invoice_title']) ? $data['invoice_title'] : 'Invoice';
		$layout		= isset($data['invoice_layout']) ? $data['invoice_layout'] : '';
		
		if (empty($ide)) {
			return false;
		}
		
		$query->clear()
			->select($db->qn('id'))
			->from($db->qn('#__rseventspro_invoice'))
			->where($db->qn('lang').' = '.$db->q($language))
			->where($db->qn('ide').' = '.$db->q($ide));
		
		$db->setQuery($query);
		if ($id = $db->loadResult()) {
			$query->clear()
				->update($db->qn('#__rseventspro_invoice'))
				->set($db->qn('title').' = '.$db->q($title))
				->set($db->qn('layout').' = '.$db->q($layout))
				->where($db->qn('id').' = '.$db->q($id));
		} else {
			$query->clear()
				->insert($db->qn('#__rseventspro_invoice'))
				->set($db->qn('layout').' = '.$db->q($layout))
				->set($db->qn('title').' = '.$db->q($title))
				->set($db->qn('lang').' = '.$db->q($language))
				->set($db->qn('ide').' = '.$db->q($ide));
		}
		
		$db->setQuery($query);
		$db->execute();
		
		return true;
	}
}