<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/
defined( '_JEXEC' ) or die( 'Restricted access' );

class RseventsproModelLocation extends JModelAdmin
{
	protected $text_prefix = 'COM_RSEVENTSPRO';

	/**
	 * Returns a Table object, always creating it.
	 *
	 * @param	type	The table type to instantiate
	 * @param	string	A prefix for the table class name. Optional.
	 * @param	array	Configuration array for model. Optional.
	 *
	 * @return	JTable	A database object
	*/
	public function getTable($type = 'Location', $prefix = 'RseventsproTable', $config = array()) {
		return JTable::getInstance($type, $prefix, $config);
	}
	
	/**
	 * Method to get a single record.
	 *
	 * @param	integer	The id of the primary key.
	 *
	 * @return	mixed	Object on success, false on failure.
	 */
	public function getItem($pk = null) {
		if ($item = parent::getItem($pk)) {
			// Convert the gallery tags field to an array.
			try {
				$registry = new JRegistry;
				$registry->loadString($item->gallery_tags ? $item->gallery_tags : '');
				$item->gallery_tags = $registry->toArray();
			} catch (Exception $e) {
				$item->gallery_tags = array();
			}
			
			RSEventsProTranslations::overrideValues($item, 'location');
		}
		
		return $item;
	}
	
	/**
	 * Method to get the record form.
	 *
	 * @param	array	$data		Data for the form.
	 * @param	boolean	$loadData	True if the form is to load its own data (default case), false if not.
	 *
	 * @return	mixed	A JForm object on success, false on failure
	 * @since	1.6
	 */
	public function getForm($data = array(), $loadData = true) {
		// Get the form.
		$form = $this->loadForm('com_rseventspro.location', 'location', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form))
			return false;
		
		if (rseventsproHelper::isJ4()) {
			$form->setFieldAttribute('gallery_tags', 'layout', 'joomla.form.field.list-fancy-select');
		}
		
		RSEventsProTranslations::setTranslateIcon($form, 'location');
		
		return $form;
	}
	
	/**
	 * Method to get the data that should be injected in the form.
	 *
	 * @return	mixed	The data for the form.
	 * @since	1.6
	 */
	protected function loadFormData() {
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_rseventspro.edit.location.data', array());

		if (empty($data))
			$data = $this->getItem();

		return $data;
	}
}