<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

// No direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

class RseventsproTableEvent extends JTable
{	
	/**
	 * @param	JDatabase	A database connector object
	 */
	public function __construct($db) {
		parent::__construct('#__rseventspro_events', 'id', $db);
	}
	
	/**
	 * Overloaded bind function
	 *
	 * @param	array		Named array
	 * @return	null|string	null is operation was satisfactory, otherwise returns an error
	 * @since	1.6
	 */
	public function bind($array, $ignore = '') {
		if (!isset($array['id']) || empty($array['id'])) {
			if ($fields = $this->getFields()) {
				foreach ($fields as $key => $field) {
					if (!isset($array[$key])) {
						if (strpos(strtolower($field->Type), 'int') !== false || strpos(strtolower($field->Type), 'float') !== false) {
							$array[$key] = 0;
						} elseif (strpos(strtolower($field->Type), 'datetime') !== false) {
							$array[$key] = JFactory::getDbo()->getNullDate();
						} else {
							$array[$key] = '';
						}
					}
				}
			}
		}
		
		return parent::bind($array, $ignore);
	}
	
	
	/**
	 * Method to perform sanity checks on the JTable instance properties to ensure
	 * they are safe to store in the database.  Child classes should override this
	 * method to make sure the data they are storing in the database is safe and
	 * as expected before storage.
	 *
	 * @return  boolean  True if the instance is sane and able to be stored in the database.
	 *
	 * @link    http://docs.joomla.org/JTable/check
	 * @since   11.1
	 */
	public function check() {
		$db		  = $this->getDbo();
		$app	  = JFactory::getApplication();
		$tzoffset = JFactory::getConfig()->get('offset');
		
		if ($this->URL == 'http://') $this->URL = '';
		
		// Manipulate dates
		if (empty($this->id)) {
			$user	= JFactory::getUser();
			$end	= JFactory::getDate();
			$end->modify('+2 hours');
			
			if ($app->isClient('administrator')) { 
				$this->published = 1;
			}
			
			if ($app->isClient('site')) {
				$permissions = rseventsproHelper::permissions();
				
				if (!empty($permissions['default_category'])) {
					try {
						$registry = new JRegistry;
						$registry->loadString($permissions['default_category']);
						$default_category = $registry->toArray();
					} catch (Exception $e) {
						$default_category = array();
					}
					
					if ($default_category) {
						$app->input->set('categories', $default_category);
					}
				}
				
				if (!empty($permissions['default_location'])) {
					$this->location = $permissions['default_location'];
				}
			}
			
			$this->name		= empty($this->name) ? JText::_('COM_RSEVENTSPRO_NEW_EVENT') : $this->name;
			$this->start	= (empty($this->start) || $this->start == $db->getNullDate()) ? JFactory::getDate()->toSql() : $this->start;
			$this->created	= JFactory::getDate()->toSql();
			
			if (!isset($this->from)) {
				$this->end = (empty($this->end) || $this->end == $db->getNullDate()) ? $end->toSql() : $this->end;
			} else {
				unset($this->from);
			}
			
			$this->owner = empty($this->owner) ? $user->get('id') : $this->owner;
			$this->options = rseventsproHelper::getDefaultOptions();
			$this->waitinglist_time = array(1,0,0);
			$this->language = JFactory::getLanguage()->getTag();
			
			if ($user->get('guest')) {
				$this->sid = JFactory::getSession()->getId();
			}
		} else {
			if ($this->allday) {
				$start = JFactory::getDate($this->start, $tzoffset);
				$start->setTimezone(new DateTimezone('UTC'));
				$this->start = $start->toSql();
				$this->end	 = $db->getNullDate();
			} else {
				$start = JFactory::getDate($this->start, $tzoffset);
				$start->setTimezone(new DateTimezone('UTC'));
				
				if ($start->format('I')) {
					$start->modify('-1 hours');
				}
				
				$this->start = $start->toSql();
				
				$end = JFactory::getDate($this->end, $tzoffset);
				$end->setTimezone(new DateTimezone('UTC'));
				
				if ($start > $end) {
					$end->modify('+2 hours');
				}
				
				if ($end->format('I')) {
					$end->modify('-1 hours');
				}
				
				$this->end = $end->toSql();
			}
			
			// Check for start date
			if (empty($this->start) || $this->start == $db->getNullDate()) {
				$this->setError(JText::_('COM_RSEVENTSPRO_PLEASE_INPUT_START_DATE'));
				return false;
			}
			
			// Check for end date
			if ((empty($this->end) || $this->start == $db->getNullDate()) && !$this->allday) {
				$this->setError(JText::_('COM_RSEVENTSPRO_PLEASE_INPUT_END_DATE'));
				return false;
			}
			
			// Check start and end dates
			if (!$this->allday) {
				if (JFactory::getDate($this->start) > JFactory::getDate($this->end)) {
					$this->setError(JText::_('COM_RSEVENTSPRO_END_BIGGER_ERROR'));
					return false;
				}
			}
			
			// Check for a location
			if (empty($this->location)) {
				$this->setError(JText::_('COM_RSEVENTSPRO_PLEASE_SELECT_LOCATION'));
				return false;
			}
			
			// Check for categories
			$categories = $app->input->get('categories',array(),'array');
			// Check for allowed categories
			if ($app->isClient('site')) {
				rseventsproHelper::allowedCategories($categories);
			}
			
			if (count($categories) == 0) {
				$this->setError(JText::_('COM_RSEVENTSPRO_PLEASE_SELECT_CATEGORY'));
				return false;
			}
			
			// Check for consent
			if ($app->isClient('site') && rseventsproHelper::getConfig('consent', 'int') && !$app->input->getInt('consent')) {
				$this->setError(JText::_('COM_RSEVENTSPRO_CONSENT_INFO'));
				return false;
			}
			
			$lang = $app->isClient('site') ? JFactory::getLanguage()->getTag() : RSEventsProTranslations::getLanguage('event', $this->id);
			RSEventsProTranslations::saveTranslations($this, 'event', $lang);
		}
		
		// Start registration
		if (!empty($this->start_registration) && $this->start_registration != $db->getNullDate()) {
			$start_registration  = JFactory::getDate($this->start_registration, $tzoffset);
			$this->start_registration = $start_registration->toSql();
		} else {
			$this->start_registration = $db->getNullDate();
		}
		
		// End registration
		if (!empty($this->end_registration) && $this->end_registration != $db->getNullDate()) {
			$end_registration  = JFactory::getDate($this->end_registration, $tzoffset);
			$this->end_registration = $end_registration->toSql();
		} else {
			$this->end_registration = $db->getNullDate();
		}
		
		// Unsubscribe date
		if (!empty($this->unsubscribe_date) && $this->unsubscribe_date != $db->getNullDate()) {
			$this->unsubscribe_date = JFactory::getDate($this->unsubscribe_date, $tzoffset)->toSql();
		} else {
			$this->unsubscribe_date = $db->getNullDate();
		}
		
		if (!empty($this->repeat_end) && $this->repeat_end != $db->getNullDate()) {
			$this->repeat_end = JFactory::getDate($this->repeat_end, $tzoffset)->toSql();
		} else {
			$this->repeat_end = $db->getNullDate();
		}
		
		if (!empty($this->rsvp_start) && $this->rsvp_start != $db->getNullDate()) {
			$this->rsvp_start = JFactory::getDate($this->rsvp_start, $tzoffset)->toSql();
		} else {
			$this->rsvp_start = $db->getNullDate();
		}
		
		if (!empty($this->rsvp_end) && $this->rsvp_end != $db->getNullDate()) {
			$this->rsvp_end = JFactory::getDate($this->rsvp_end, $tzoffset)->toSql();
		} else {
			$this->rsvp_end = $db->getNullDate();
		}
		
		// Discounts
		if ($this->discounts) {
			if ($this->early_fee_end && $this->early_fee_end != $db->getNullDate()) {
				$this->early_fee_end = JFactory::getDate($this->early_fee_end, $tzoffset)->toSql();
			} else {
				$this->early_fee_end = $db->getNullDate();
			}

			if ($this->late_fee_start && $this->late_fee_start != $db->getNullDate()) {
				$this->late_fee_start = JFactory::getDate($this->late_fee_start, $tzoffset)->toSql();
			} else {
				$this->late_fee_start = $db->getNullDate();
			}
		} else {
			$this->early_fee_end = $db->getNullDate();
			$this->late_fee_start = $db->getNullDate();
		}
		
		// Repeat dates
		if (isset($this->repeat_also) && is_array($this->repeat_also)) {
			$dates = array_unique($this->repeat_also);
			$dates = array_merge($dates,array());
			
			$registry = new JRegistry();
			$registry->loadArray($dates);
			$this->repeat_also = (string) $registry;
		} else $this->repeat_also = '';
		
		// Exclude dates
		if (isset($this->exclude_dates) && is_array($this->exclude_dates)) {
			$exclude = array_unique($this->exclude_dates);
			$exclude = array_merge($exclude,array());
			
			$registry = new JRegistry();
			$registry->loadArray($exclude);
			$this->exclude_dates = (string) $registry;
		} else $this->exclude_dates = '';
		
		if (isset($this->payments) && is_array($this->payments)) {
			$registry = new JRegistry();
			$registry->loadArray($this->payments);
			$this->payments = (string) $registry;
		} else $this->payments = '';
		
		if (isset($this->gallery_tags) && is_array($this->gallery_tags)) {
			$registry = new JRegistry;
			$registry->loadArray($this->gallery_tags);
			$this->gallery_tags = (string) $registry;
		} else {
			$this->gallery_tags = '';
		}
		
		if (isset($this->rsm_lists) && is_array($this->rsm_lists)) {
			$registry = new JRegistry;
			$registry->loadArray($this->rsm_lists);
			$this->rsm_lists = (string) $registry;
		} else {
			$this->rsm_lists = '';
		}
		
		if (isset($this->waitinglist_time) && is_array($this->waitinglist_time)) {
			$this->waitinglist_time = intval($this->waitinglist_time[0]) * 86400 + intval($this->waitinglist_time[1]) * 3600 + intval($this->waitinglist_time[2]) * 60;
		} else {
			$this->waitinglist_time = '';
		}
		
		$updateOptions = true;
		
		if ($app->isClient('site')) {
			$permissions = rseventsproHelper::permissions();
			
			if (empty($permissions['can_change_options'])) {
				$updateOptions = false;
			}
		}
		
		if ($updateOptions) {		
			if (isset($this->options) && is_array($this->options)) {
				$registry = new JRegistry();
				$registry->loadArray($this->options);
				$this->options = (string) $registry;
			} else $this->options = '';
		}

		$this->timezone = $tzoffset;
		
		if (empty($this->itemid)) $this->itemid = 0;
		if (empty($this->max_tickets_amount)) $this->max_tickets_amount = 0;
		if (empty($this->overbooking_amount)) $this->overbooking_amount = 0;
		if (empty($this->waitinglist_limit)) $this->waitinglist_limit = 0;
		if (empty($this->tickets_amount)) $this->tickets_amount = 0;
		if (empty($this->rsvp_quota)) $this->rsvp_quota = 0;
		if (empty($this->invoice_padding)) $this->invoice_padding = 0;
		
		return true;
	}
	
	
	/**
	 * Method to set the publishing state for a row or list of rows in the database
	 * table.  The method respects checked out rows by other users and will attempt
	 * to checkin rows that it can after adjustments are made.
	 *
	 * @param   mixed    $pks     An optional array of primary key values to update.  If not set the instance property value is used.
	 * @param   integer  $state   The publishing state. eg. [0 = unpublished, 1 = published]
	 * @param   integer  $userId  The user id of the user performing the operation.
	 *
	 * @return  boolean  True on success.
	 *
	 * @link    http://docs.joomla.org/JTable/publish
	 * @since   11.1
	 */
	public function publish($pks = null, $value = 1, $userid = 0) {
		$db		= JFactory::getDBO();
		$query	= $db->getQuery(true);
		$task	= JFactory::getApplication()->input->getCmd('task');
		
		if (count($pks) == 1 && $task == 'unpublish') {
			$query->clear()
				->select($db->qn('published'))
				->from($db->qn('#__rseventspro_events'))
				->where($db->qn('id').' = '.(int) @$pks[0]);
			$db->setQuery($query);
			$state = (int) $db->loadResult();
			if ($state == 2) {
				JFactory::getApplication()->enqueueMessage(JText::_('COM_RSEVENTSPRO_ARCHIVE_INFO'));
			}
		}
		
		if ($task == 'archive') {
			$query->clear()
				->update($db->qn('#__rseventspro_events'))
				->set($db->qn('archived'). ' = '.$db->q(1))
				->where($db->qn('id'). ' IN ('.implode(',',$pks).')');
			
			$db->setQuery($query);
			$db->execute();
		} else {
			if ($value == 1) {
				$query->clear()
					->update($db->qn('#__rseventspro_events'))
					->set($db->qn('approved'). ' = '.$db->q(0))
					->where($db->qn('id'). ' IN ('.implode(',',$pks).')');
				
				$db->setQuery($query);
				$db->execute();
			}
		}
		
		return parent::publish($pks, $value, $userid);
	}
	
	/**
	 * Method to delete a node and, optionally, its child nodes from the table.
	 *
	 * @param   integer  $pk        The primary key of the node to delete.
	 * @param   boolean  $children  True to delete child nodes, false to move them up a level.
	 *
	 * @return  boolean  True on success.
	 *
	 * @see     http://docs.joomla.org/JTable/delete
	 * @since   2.5
	 */
	public function delete($pk = null, $children = false) {
		$delete = rseventsproHelper::remove($pk);
		
		if ($delete) RSEventsProTranslations::removeTranslations('event', $pk);
		
		return $delete;
	}
	
	public function verify(&$array) {
		if (!isset($array['recurring'])) 				$array['recurring'] = 0;
		if (!isset($array['allday']))					$array['allday'] = 0;
		if (!isset($array['discounts']))				$array['discounts'] = 0;
		if (!isset($array['ticketsconfig']))			$array['ticketsconfig'] = 0;
		if (!isset($array['registration']))				$array['registration'] = 0;
		if (!isset($array['rsvp']))						$array['rsvp'] = 0;
		if (!isset($array['rsvp_guests']))				$array['rsvp_guests'] = 0;
		if (!isset($array['rsvp_going']))				$array['rsvp_going'] = 0;
		if (!isset($array['rsvp_interested']))			$array['rsvp_interested'] = 0;
		if (!isset($array['rsvp_notgoing']))			$array['rsvp_notgoing'] = 0;
		if (!isset($array['comments']))					$array['comments'] = 0;
		if (!isset($array['notify_me']))				$array['notify_me'] = 0;
		if (!isset($array['notify_me_paid']))			$array['notify_me_paid'] = 0;
		if (!isset($array['waitinglist_user']))			$array['waitinglist_user'] = 0;
		if (!isset($array['waitinglist_admin']))		$array['waitinglist_admin'] = 0;
		if (!isset($array['notify_me_unsubscribe']))	$array['notify_me_unsubscribe'] = 0;
		if (!isset($array['overbooking']))				$array['overbooking'] = 0;
		if (!isset($array['max_tickets']))				$array['max_tickets'] = 0;
		if (!isset($array['waitinglist']))				$array['waitinglist'] = 0;
		if (!isset($array['show_registered']))			$array['show_registered']= 0;
		if (!isset($array['automatically_approve']))	$array['automatically_approve'] = 0;
		if (!isset($array['invoice']))					$array['invoice'] = 0;
		if (!isset($array['invoice_attach']))			$array['invoice_attach'] = 0;
		if (!isset($array['disable_registration']))		$array['disable_registration'] = 0;
		if (!isset($array['disable_activation']))		$array['disable_activation'] = 0;
		if (!isset($array['disable_denied']))			$array['disable_denied'] = 0;
		if (!isset($array['disable_refund']))			$array['disable_refund'] = 0;
		
		if (isset($array['options'])) {
			$defaults = rseventsproHelper::getDefaultOptions();
			try {
				$registry = new JRegistry;
				$registry->loadString($defaults);
				$defaults = $registry->toArray();
			} catch (Exception $e) {}
			
			foreach ($defaults as $name => $value) {
				if (!isset($array['options'][$name]))
					$array['options'][$name] = 0;
			}
		}
		
		return true;
	}
}