<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2015 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

// no direct access
defined('_JEXEC') or die('Restricted access');

global $_PLUGINS;
$_PLUGINS->registerFunction( 'onBeforegetFieldRow', 'onBeforegetFieldRow', 'getRseproTab' );
$_PLUGINS->registerUserFieldTypes( array('rseprostatus'	=> 'CBfield_rseprostatus') );
$_PLUGINS->registerUserFieldParams();

class CBfield_rseprostatus extends cbFieldHandler {
	
	/**
	 * Accessor:
	 * Returns a field in specified format
	 *
	 * @param  moscomprofilerFields  $field
	 * @param  moscomprofilerUser    $user
	 * @param  string                $output  'html', 'xml', 'json', 'php', 'csvheader', 'csv', 'rss', 'fieldslist', 'htmledit'
	 * @param  string                $reason  'profile' for user profile view, 'edit' for profile edit, 'register' for registration, 'search' for searches, 'list' for user-lists
	 * @param  int                   $list_compare_types   IF reason == 'search' : 0 : simple 'is' search, 1 : advanced search with modes, 2 : simple 'any' search
	 * @return mixed                
	 */
	public function getField( &$field, &$user, $output, $reason, $list_compare_types ) {
		global $ueConfig;
		
		static $stats =	array();
		$value        =	null;

		if (is_object($user) && $user->id ) {
			if (!isset($stats[$user->id])) {
				global $_PLUGINS;
				$params								=	array( &$user );
				$stats[$user->id]					=	$_PLUGINS->call( $this->getPluginId(), '', 'getRseproTab', $params );
			}
			if ( isset( $stats[$user->id][$field->name] ) ) {
				$value								=	$stats[$user->id][$field->name];
			} else {
				$value								=	$ueConfig['emptyFieldsText'];
			}
		}

		switch ( $output ) {
			case 'html':
			case 'rss':
				return $this->_formatFieldOutput( $field->name, $value, $output, false );
			break;
			case 'htmledit':
				return null;
			break;
			default:
				return $this->_formatFieldOutput( $field->name, $value, $output, false );
			break;
		}
	}
}

class getRseproTab extends cbTabHandler {
	
	function __construct() {
		$this->cbTabHandler();
	}
	
	/**
	* Generates the HTML to display the user profile tab
	* @param  moscomprofilerTab   $tab       the tab database entry
	* @param  moscomprofilerUser  $user      the user being displayed
	* @param  int                 $ui        1 for front-end, 2 for back-end
	* @return mixed                          either string HTML for tab content, or false if ErrorMSG generated
	*/
	public function getDisplayTab( $tab, $user, $ui ) {
		if (!file_exists(JPATH_SITE.'/components/com_rseventspro/helpers/rseventspro.php')) return;
		
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/rseventspro.php';
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/route.php';
		
		$params		= $this->params;
		$limit 		= $params->get('limit',5);
		$ordering	= $params->get('ordering','ASC');
		$target		= $params->get('target',1);
		$opener		= $target ? '' : 'target="_blank"';
		$itemid		= $params->get('itemid','');
		$itemid		= empty($itemid) ? RseventsproHelperRoute::getEventsItemid() : (int) $itemid;
		
		//load language and style
		JFactory::getLanguage()->load('com_rseventspro');
		JFactory::getDocument()->addStyleSheet(JURI::root().'components/com_comprofiler/plugin/user/plug_rseventspro/style.css');
		
		//get events
		$return 	= '<div class="rsepro_events_container">';
		$events 	= $this->_getEvents($user->id,$limit,$ordering);
		$booked		= $this->_getBookedEvents($user->id,$limit);
		
		if (empty($events) && empty($booked)) return '<div>'.JText::_('RSEPRO_CB_NO_EVENTS').'</div>';
		
		if ($events) {
			$return .= '<h3>'.JText::_('COM_RSEVENTSPRO_CB_MY_EVENTS').'</h3>';
			$return .= '<ul class="rsepro_events">';
			foreach ($events as $eid) {
				$details = rseventsproHelper::details($eid);
				if (isset($details['event']) && !empty($details['event'])) $event = $details['event']; else continue;
				
				if ($event->allday)
					$return .= '<li><a '.$opener.' href="'.rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),true,$itemid).'">'.$event->name.'</a> ('.rseventsproHelper::showdate($event->start,rseventsproHelper::getConfig('global_date'),true).')</li>';
				else
					$return .= '<li><a '.$opener.' href="'.rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),true,$itemid).'">'.$event->name.'</a> ('.rseventsproHelper::showdate($event->start,null,true).' - '.rseventsproHelper::showdate($event->end,null,true).')</li>';
			}
			$return .= '</ul>';
		}
		
		if ($booked) {
			$return .= '<h3>'.JText::_('COM_RSEVENTSPRO_CB_MY_SUBSCRIPTIONS').'</h3>';
			$return .= '<ul class="rsepro_events">';
			foreach ($booked as $eid) {
				$details = rseventsproHelper::details($eid);
				if (isset($details['event']) && !empty($details['event'])) $event = $details['event']; else continue;
				
				if ($event->allday)
					$return .= '<li><a '.$opener.' href="'.rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),true,$itemid).'">'.$event->name.'</a> ('.rseventsproHelper::showdate($event->start,rseventsproHelper::getConfig('global_date'),true).')</li>';
				else
					$return .= '<li><a '.$opener.' href="'.rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),true,$itemid).'">'.$event->name.'</a> ('.rseventsproHelper::showdate($event->start,null,true).' - '.rseventsproHelper::showdate($event->end,null,true).')</li>';
			}
			$return .= '</ul>';
		}
		
		$return .= '</div>';
		
		return $return;
	}
	
	
	protected function _getEvents($user, $limit, $order) {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$temp	= array();
		
		$query->clear()
			->select($db->qn('id'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('published').' = 1')
			->where($db->qn('completed').' = 1')
			->where($db->qn('owner').' = '.(int) $user)
			->order($db->qn('start').' '.$db->escape($order));
		
		$db->setQuery($query);
		$events = $db->loadColumn();
		
		if (!empty($events)) {
			$i = 0;
			foreach ($events as $eid) {
				if (!rseventsproHelper::canview($eid)) continue;
				if ($i >= $limit) break;
				$temp[] = $eid;
				$i++;
			}
		}
		
		return $temp;
	}
	
	protected function _getBookedEvents($user, $limit) {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$temp	= array();
		$events	= array();
		
		if (JPluginHelper::isEnabled('system', 'rseprocart') && file_exists(JPATH_SITE.'/plugins/system/rseprocart/rseprocart.php')) {
			$query->clear()
				->select($db->qn('u.ide'))->select($db->qn('c.events'))
				->from($db->qn('#__rseventspro_users', 'u'))
				->join('LEFT', $db->qn('#__rseventspro_cart','c').' ON '.$db->qn('u.id').' = '.$db->qn('c.ids'))
				->where($db->qn('u.idu').' = '.(int) $user)
				->order($db->qn('u.date').' DESC');
			$db->setQuery($query);
			if ($data = $db->loadObjectList()) {
				foreach ($data as $subscription) {
					// Standard registration
					if (!empty($subscription->ide)) {
						$events[] = $subscription->ide;
					}
					
					// Cart registration
					if (!empty($subscription->events)) {
						if ($ids = explode(',', $subscription->events)) {
							foreach ($ids as $id) {
								$events[] = $id;
							}
						}
					}
				}
				
				if ($events) {
					$events = array_unique($events);
				}
			}
		} else {
			$query->clear()
				->select('DISTINCT('.$db->qn('ide').')')
				->from($db->qn('#__rseventspro_users'))
				->where($db->qn('idu').' = '.(int) $user)
				->order($db->qn('date').' DESC');
			$db->setQuery($query);
			$events = $db->loadColumn();
		}
		
		if (!empty($events)) {
			$i = 0;
			foreach ($events as $eid) {
				if (!rseventsproHelper::canview($eid)) continue;
				if ($i >= $limit) break;
				$temp[] = $eid;
				$i++;
			}
		}
		
		return $temp;
	}
}