<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2022 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/
defined( '_JEXEC' ) or die( 'Restricted access' );

class RSEventsProTranslations
{
	public static function getLanguage($view = null, $id = null) {
		$lang		= JFactory::getLanguage();
		$session	= JFactory::getSession();
		$input		= JFactory::getApplication()->input;
		$view		= $view ? $view : $input->get('view','');
		$id			= $id ? $id : $input->getInt('id', 0);
		
		return $session->get('com_rseventspro.'.$view.'.id'.$id.'.lang', $lang->getTag());
	}
	
	public static function getTranslations($reference, $id, $lang = null, $select = 'value') {
		if (!JLanguageMultilang::isEnabled()) {
			return false;
		}
		
		if (is_null($lang)) {
			$lang = JFactory::getLanguage()->getTag();
		}
		
		static $selections = array();
		
		$hash = md5($reference.$id.$lang.$select);
		
		if (!isset($selections[$hash])) {
			$selections[$hash] = array();
			
			$db		= JFactory::getDbo();
			$query	= $db->getQuery(true);
			
			$query->clear()
				->select('*')
				->from($db->qn('#__rseventspro_translations'))
				->where($db->qn('reference').' = '.$db->q($reference))
				->where($db->qn('reference_id').' = '.$db->q($id))
				->where($db->qn('language').' = '.$db->q($lang));
			$db->setQuery($query);
			
			if ($results = $db->loadObjectList()) {
				foreach ($results as $result) {
					$selections[$hash][$result->property] = ($select == '*') ? $result : (isset($result->$select) ? $result->$select : false);
				}
			}
		}
		
		return $selections[$hash];
	}
	
	public static function getTranslation($reference, $id, $property, $lang = null) {
		if (!JLanguageMultilang::isEnabled()) {
			return false;
		}
		
		if (is_null($lang)) {
			$lang = JFactory::getLanguage()->getTag();
		}
		
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('value'))
			->from($db->qn('#__rseventspro_translations'))
			->where($db->qn('reference').' = '.$db->q($reference))
			->where($db->qn('reference_id').' = '.$db->q($id))
			->where($db->qn('language').' = '.$db->q($lang))
			->where($db->qn('property').' = '.$db->q($property));
		$db->setQuery($query);
		
		return $db->loadResult();
	}
	
	public static function saveTranslations(&$form = null, $reference = null, $lang = null) {
		if (!JLanguageMultilang::isEnabled()) {
			return false;
		}
		
		if ($form->language == $lang) {
            return true;
        }
		
		$db				= JFactory::getDbo();
		$fields			= self::getFields($reference);
		$translations	= self::getTranslations($reference, $form->id, $lang, 'id');
		
		foreach ($fields as $field) {
            $translation = (object) array(
                'language'		=> $lang,
                'reference'		=> $reference,
                'reference_id'	=> $form->id,
                'property'		=> $field,
                'value'			=> $form->{$field}
            );

			if (!isset($translations[$field])) {
			    $db->insertObject('#__rseventspro_translations', $translation);
			} else {
			    $translation->id = $translations[$field];
				$db->updateObject('#__rseventspro_translations', $translation, array('id'));
			}
			
			unset($form->$field);
		}
		
		return true;
	}
	
	public static function showLanguageBox($lang = null, $frontend = false) {
		if (JLanguageMultilang::isEnabled()) {
			$html		= array();
			$lang		= $lang ? $lang : self::getLanguage();
			
			if ($frontend) {
				$html[] = '<div class="alert alert-info">';
				$html[] = JText::sprintf('COM_RSEVENTSPRO_MULTILANGUAGE_TEXT', $lang);
				$html[] = '</div>';
			} else {
				$options	= array();
				$languages	= JLanguageHelper::getKnownLanguages(JPATH_SITE);
				
				foreach ($languages as $language => $properties) {
					$options[] = JHtml::_('select.option', $language, $properties['name']);
				}
				
				$listoptions = array();
				$listoptions['option.key'] = 'value';
				$listoptions['option.text'] = 'text';
				$listoptions['list.select'] = $lang;
				$listoptions['id'] = 'rsepro-lang';
				$listoptions['list.translate'] = false;
				$listoptions['option.attr'] = 'optionattr';
				$listoptions['list.attr'] = 'class="form-control" onchange="Joomla.submitform(\'changelanguage\')"';
				
				$html[] = '<div class="alert alert-info">';
				$html[] = JHtml::_('select.genericlist', $options, 'rseprolang', $listoptions);
				$html[] = JText::sprintf('COM_RSEVENTSPRO_MULTILANGUAGE_TEXT', $lang);
				$html[] = '</div>';
			}
			
			return implode("\n", $html);
		}
	}
	
	public static function setTranslateIcon(&$form, $reference) {
		if (JLanguageMultilang::isEnabled()) {
			if ($fields = self::getFields($reference)) {
				foreach ($fields as $field) {
					$label = '<i class="fa fa-flag"></i> ';
					$label .= JText::_($form->getFieldAttribute($field, 'label'));
					
					$form->setFieldAttribute($field, 'label', $label);
				}
			}
		}
	}
	
	public static function overrideValues(&$item, $reference) {
		if (JLanguageMultilang::isEnabled()) {
			
			if (strpos($reference, '.') !== false) {
				list($referenceLanguage, $referenceLanguageID, $reference, $referenceID) = explode('.', $reference);
			} else {
				$referenceLanguage = $reference;
				$referenceLanguageID = $item->id;
				$referenceID = $item->id;
			}
			
			$lang = self::getLanguage($referenceLanguage, $referenceLanguageID);
			
			if ($item->language != $lang) {
				if ($translations = self::getTranslations($reference, $referenceID, $lang)) {
					foreach ($translations as $field => $value) {
						if (isset($item->{$field})) {
							$item->{$field} = $value;
						}
					}
				}
			}
		}
	}
	
	public static function removeTranslations($reference, $id) {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->delete($db->qn('#__rseventspro_translations'))
			->where($db->qn('reference').' = '.$db->q($reference))
			->where($db->qn('reference_id').' = '.$db->q($id));
		$db->setQuery($query);
		$db->execute();
	}
	
	protected static function getFields($reference) {
		if (!JLanguageMultilang::isEnabled()) {
			return false;
		}
		
		if ($reference == 'location') {
			return array('name', 'description');
		} elseif ($reference == 'ticket') {
			return array('name', 'layout', 'description', 'ticketinfolayout');
		} elseif ($reference == 'tag') {
			return array('name');
		} elseif ($reference == 'payment') {
			return array('name', 'details');
		} elseif ($reference == 'speaker' || $reference == 'user') {
			return array('description');
		} elseif ($reference == 'category') {
			return array('title', 'description', 'metadesc', 'metakey');
		} elseif ($reference == 'event') {
			return array('name', 'small_description', 'description', 'URL', 'phone', 'email', 'metaname', 'metakeywords', 'metadescription', 'event_ended', 'event_full');
		} elseif ($reference == 'field') {
			return array('label', 'description', 'validationMessage', 'value');
		}
	}
}